"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WinixPurifierAccessory = void 0;
const winix_api_1 = require("winix-api");
const MIN_AMBIENT_LIGHT = 0.0001;
class WinixPurifierAccessory {
    constructor(log, config, api) {
        var _a, _b, _c, _d;
        this.hap = api.hap;
        this.log = log;
        const deviceName = config.name;
        this.deviceId = config.deviceId;
        this.latestStatus = {};
        this.cacheIntervalMs = config.cacheIntervalSeconds * 1000 || 60000;
        this.lastWinixPoll = -1;
        this.services = [];
        // Create services
        this.purifier = this.registerService(new this.hap.Service.AirPurifier(deviceName));
        this.purifier.setPrimaryService(true);
        const purifierInfo = this.registerService(new this.hap.Service.AccessoryInformation());
        if (config.exposeAirQuality) {
            this.airQuality = this.registerService(new this.hap.Service.AirQualitySensor('Air Quality', 'air-quality-sensor')
                .setCharacteristic(this.hap.Characteristic.ConfiguredName, 'Air Quality'));
        }
        if (config.exposePlasmawave) {
            this.plasmawave = this.registerService(new this.hap.Service.Switch('Plasmawave', 'switch-plasmawave')
                .setCharacteristic(this.hap.Characteristic.ConfiguredName, 'Plasmawave'));
        }
        if (config.exposeAmbientLight) {
            this.ambientLight = this.registerService(new this.hap.Service.LightSensor('Ambient Light', 'light-sensor-ambient')
                .setCharacteristic(this.hap.Characteristic.ConfiguredName, 'Ambient Light'));
        }
        if (config.exposeAutoSwitch) {
            this.autoSwitch = this.registerService(new this.hap.Service.Switch('Auto Mode', 'switch-auto')
                .setCharacteristic(this.hap.Characteristic.ConfiguredName, 'Auto Mode'));
        }
        // Assign characteristics
        this.purifier.getCharacteristic(this.hap.Characteristic.Active)
            .onGet(this.getActiveState.bind(this))
            .onSet(this.setActiveState.bind(this));
        this.purifier.getCharacteristic(this.hap.Characteristic.CurrentAirPurifierState)
            .onGet(this.getCurrentState.bind(this));
        this.purifier.getCharacteristic(this.hap.Characteristic.TargetAirPurifierState)
            .onGet(this.getTargetState.bind(this))
            .onSet(this.setTargetState.bind(this));
        this.purifier.getCharacteristic(this.hap.Characteristic.RotationSpeed)
            .onGet(this.getRotationSpeed.bind(this))
            .onSet(this.setRotationSpeed.bind(this));
        purifierInfo
            .setCharacteristic(this.hap.Characteristic.Manufacturer, 'Winix')
            .setCharacteristic(this.hap.Characteristic.SerialNumber, config.serialNumber)
            .setCharacteristic(this.hap.Characteristic.Model, config.model);
        (_a = this.airQuality) === null || _a === void 0 ? void 0 : _a.getCharacteristic(this.hap.Characteristic.AirQuality).onGet(this.getAirQuality.bind(this));
        (_b = this.plasmawave) === null || _b === void 0 ? void 0 : _b.getCharacteristic(this.hap.Characteristic.On).onGet(this.getPlasmawave.bind(this)).onSet(this.setPlasmawave.bind(this));
        (_c = this.ambientLight) === null || _c === void 0 ? void 0 : _c.getCharacteristic(this.hap.Characteristic.CurrentAmbientLightLevel).onGet(this.getAmbientLight.bind(this));
        (_d = this.autoSwitch) === null || _d === void 0 ? void 0 : _d.getCharacteristic(this.hap.Characteristic.On).onGet(this.getAutoSwitchState.bind(this)).onSet(this.setAutoSwitchState.bind(this));
    }
    async getActiveState() {
        if (this.shouldUseCachedValue(this.latestStatus.power)) {
            this.log.debug('getActiveState() (cached)', this.latestStatus.power);
            return this.toActiveState(this.latestStatus.power);
        }
        let power;
        try {
            power = await winix_api_1.WinixAPI.getPower(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting active state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.power = power;
        this.polledWinix();
        this.log.debug('getActiveState()', power);
        return this.toActiveState(power);
    }
    async setActiveState(state) {
        const power = state === this.hap.Characteristic.Active.ACTIVE ? winix_api_1.Power.On : winix_api_1.Power.Off;
        this.log.debug(`setActiveState(${state})`, power);
        if (this.latestStatus.power === power) {
            this.log.debug('ignoring duplicate state set: active');
            return;
        }
        try {
            await winix_api_1.WinixAPI.setPower(this.deviceId, power);
        }
        catch (e) {
            assertError(e);
            this.log.error('error setting active state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.power = power;
        this.sendHomekitUpdate();
    }
    async getCurrentState() {
        if (this.shouldUseCachedValue(this.latestStatus.power)) {
            this.log.debug('getCurrentState() (cached)', this.latestStatus.power);
            return this.toCurrentState(this.latestStatus.power);
        }
        let power;
        try {
            power = await winix_api_1.WinixAPI.getPower(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting current state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.power = power;
        this.polledWinix();
        this.log.debug('getCurrentState()', power);
        return this.toCurrentState(power);
    }
    async getAutoSwitchState() {
        const targetState = await this.getTargetState();
        this.log.debug('getAutoSwitchState() targetState', targetState);
        // Translate target state (auto/manual mode) to auto switch state
        const result = targetState === this.hap.Characteristic.TargetAirPurifierState.AUTO ?
            this.hap.Characteristic.Active.ACTIVE : this.hap.Characteristic.Active.INACTIVE;
        this.log.debug('getAutoSwitchState() result', result);
        return result;
    }
    async setAutoSwitchState(state) {
        // Translate auto switch state to target state (auto/manual mode)
        const proxyState = state ?
            this.hap.Characteristic.TargetAirPurifierState.AUTO :
            this.hap.Characteristic.TargetAirPurifierState.MANUAL;
        this.log.debug(`setAutoSwitchState(${state}) proxyState`, proxyState);
        return this.setTargetState(proxyState);
    }
    async getTargetState() {
        if (this.shouldUseCachedValue(this.latestStatus.mode)) {
            this.log.debug('getTargetState() (cached)', this.latestStatus.mode);
            return this.toTargetState(this.latestStatus.mode);
        }
        let mode;
        try {
            mode = await winix_api_1.WinixAPI.getMode(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting target state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.mode = mode;
        this.polledWinix();
        this.log.debug('getTargetState()', mode);
        return this.toTargetState(mode);
    }
    async setTargetState(state) {
        const mode = state === this.hap.Characteristic.TargetAirPurifierState.AUTO ? winix_api_1.Mode.Auto : winix_api_1.Mode.Manual;
        this.log.debug(`setTargetState(${state})`, mode);
        // Don't try to set the mode if we're already in this mode
        // Fixes issues with this being set right around the time of power on
        if (this.latestStatus.mode === mode) {
            this.log.debug('ignoring duplicate state set: target');
            return;
        }
        try {
            await winix_api_1.WinixAPI.setMode(this.deviceId, mode);
        }
        catch (e) {
            assertError(e);
            this.log.error('error setting target state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.mode = mode;
        this.sendHomekitUpdate();
        if (mode === winix_api_1.Mode.Manual) {
            return;
        }
        // If we're switching back to auto, the airflow speed will most likely change on the Winix device itself.
        // Pause, get the latest airflow speed, then send the update to Homekit
        this.log.debug('scheduling homekit update to rotation speed');
        setTimeout(async () => {
            await this.getRotationSpeed(true);
            this.sendHomekitUpdate();
        }, 2000);
    }
    async getRotationSpeed(force = false) {
        if (!force && this.shouldUseCachedValue(this.latestStatus.airflow)) {
            this.log.debug('getRotationSpeed() (cached)', this.latestStatus.airflow);
            return this.toRotationSpeed(this.latestStatus.airflow);
        }
        let airflow;
        try {
            airflow = await winix_api_1.WinixAPI.getAirflow(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting rotation speed: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.airflow = airflow;
        this.polledWinix();
        this.log.debug('getRotationSpeed():', airflow);
        return this.toRotationSpeed(airflow);
    }
    async setRotationSpeed(state) {
        const airflow = this.toAirflow(state);
        this.log.debug(`setRotationSpeed(${state}):`, airflow);
        try {
            await winix_api_1.WinixAPI.setAirflow(this.deviceId, airflow);
        }
        catch (e) {
            assertError(e);
            this.log.error('error setting rotation speed: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.airflow = airflow;
        this.latestStatus.mode = winix_api_1.Mode.Manual;
        this.sendHomekitUpdate();
    }
    async getAirQuality() {
        if (this.shouldUseCachedValue(this.latestStatus.airQuality)) {
            this.log.debug('getAirQuality() (cached)', this.latestStatus.airQuality);
            return this.toAirQuality(this.latestStatus.airQuality);
        }
        let airQuality;
        try {
            airQuality = await winix_api_1.WinixAPI.getAirQuality(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting air quality: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.airQuality = airQuality;
        this.polledWinix();
        this.log.debug('getAirQuality():', airQuality);
        return this.toAirQuality(airQuality);
    }
    async getPlasmawave() {
        if (this.shouldUseCachedValue(this.latestStatus.plasmawave)) {
            this.log.debug('getPlasmawave() (cached)', this.latestStatus.plasmawave);
            return this.toSwitch(this.latestStatus.plasmawave);
        }
        let plasmawave;
        try {
            plasmawave = await winix_api_1.WinixAPI.getPlasmawave(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting plasmawave state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.plasmawave = plasmawave;
        this.polledWinix();
        this.log.debug('getPlasmawave():', plasmawave);
        return this.toSwitch(plasmawave);
    }
    async setPlasmawave(state) {
        const plasmawave = this.toPlasmawave(state);
        this.log.debug(`setPlasmawave(${state}):`, plasmawave);
        try {
            await winix_api_1.WinixAPI.setPlasmawave(this.deviceId, plasmawave);
        }
        catch (e) {
            assertError(e);
            this.log.error('error setting plasmawave state: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        this.latestStatus.plasmawave = plasmawave;
        this.sendHomekitUpdate();
    }
    async getAmbientLight() {
        if (this.shouldUseCachedValue(this.latestStatus.ambientLight)) {
            this.log.debug('getAmbientLight() (cached)', this.latestStatus.ambientLight);
            return this.latestStatus.ambientLight;
        }
        let ambientLight;
        try {
            ambientLight = await winix_api_1.WinixAPI.getAmbientLight(this.deviceId);
        }
        catch (e) {
            assertError(e);
            this.log.error('error getting ambient light: ' + e.message);
            throw new this.hap.HapStatusError(-70402 /* HAPStatus.SERVICE_COMMUNICATION_FAILURE */);
        }
        // Fix ambient light value under 0.0001 warning
        ambientLight = Math.max(ambientLight, MIN_AMBIENT_LIGHT);
        this.latestStatus.ambientLight = ambientLight;
        this.polledWinix();
        this.log.debug('getAmbientLight():', ambientLight);
        return ambientLight;
    }
    sendHomekitUpdate() {
        var _a, _b, _c, _d;
        this.log.debug('sendHomekitUpdate()');
        if (this.latestStatus.power !== undefined) {
            this.purifier.updateCharacteristic(this.hap.Characteristic.Active, this.toActiveState(this.latestStatus.power));
            this.purifier.updateCharacteristic(this.hap.Characteristic.CurrentAirPurifierState, this.toCurrentState(this.latestStatus.power));
        }
        if (this.latestStatus.mode !== undefined) {
            this.purifier.updateCharacteristic(this.hap.Characteristic.TargetAirPurifierState, this.toTargetState(this.latestStatus.mode));
        }
        if (this.latestStatus.airflow !== undefined) {
            this.purifier.updateCharacteristic(this.hap.Characteristic.RotationSpeed, this.toRotationSpeed(this.latestStatus.airflow));
        }
        if (this.airQuality !== undefined && this.latestStatus.airQuality !== undefined) {
            (_a = this.airQuality) === null || _a === void 0 ? void 0 : _a.updateCharacteristic(this.hap.Characteristic.AirQuality, this.toAirQuality(this.latestStatus.airQuality));
        }
        if (this.plasmawave !== undefined && this.latestStatus.plasmawave !== undefined) {
            (_b = this.plasmawave) === null || _b === void 0 ? void 0 : _b.updateCharacteristic(this.hap.Characteristic.On, this.toSwitch(this.latestStatus.plasmawave));
        }
        if (this.ambientLight !== undefined && this.latestStatus.ambientLight !== undefined) {
            (_c = this.ambientLight) === null || _c === void 0 ? void 0 : _c.updateCharacteristic(this.hap.Characteristic.CurrentAmbientLightLevel, this.latestStatus.ambientLight);
        }
        if (this.autoSwitch !== undefined && this.latestStatus.mode !== undefined) {
            (_d = this.autoSwitch) === null || _d === void 0 ? void 0 : _d.updateCharacteristic(this.hap.Characteristic.On, this.toTargetState(this.latestStatus.mode) === this.hap.Characteristic.TargetAirPurifierState.AUTO);
        }
    }
    toActiveState(power) {
        switch (power) {
            case winix_api_1.Power.On:
                return this.hap.Characteristic.Active.ACTIVE;
            case winix_api_1.Power.Off:
                return this.hap.Characteristic.Active.INACTIVE;
        }
    }
    toCurrentState(power) {
        switch (power) {
            case winix_api_1.Power.On:
                return this.hap.Characteristic.CurrentAirPurifierState.PURIFYING_AIR;
            case winix_api_1.Power.Off:
                return this.hap.Characteristic.CurrentAirPurifierState.INACTIVE;
        }
    }
    toTargetState(mode) {
        switch (mode) {
            case winix_api_1.Mode.Auto:
                return this.hap.Characteristic.TargetAirPurifierState.AUTO;
            case winix_api_1.Mode.Manual:
                return this.hap.Characteristic.TargetAirPurifierState.MANUAL;
        }
    }
    toRotationSpeed(airflow) {
        switch (airflow) {
            case winix_api_1.Airflow.Sleep:
                return 0;
            case winix_api_1.Airflow.Low:
                return 25;
            case winix_api_1.Airflow.Medium:
                return 50;
            case winix_api_1.Airflow.High:
                return 75;
            case winix_api_1.Airflow.Turbo:
                return 100;
        }
    }
    toAirflow(state) {
        // Round to nearest 25
        const nearestState = Math.round(state / 25) * 25;
        this.log.debug(`toAirflow(${state}): ${nearestState}`);
        switch (nearestState) {
            case 0:
                return winix_api_1.Airflow.Sleep;
            case 25:
                return winix_api_1.Airflow.Low;
            case 50:
                return winix_api_1.Airflow.Medium;
            case 75:
                return winix_api_1.Airflow.High;
            case 100:
                return winix_api_1.Airflow.Turbo;
        }
        return nearestState > 100 ? winix_api_1.Airflow.Turbo : winix_api_1.Airflow.Sleep;
    }
    toAirQuality(airQuality) {
        switch (airQuality) {
            case winix_api_1.AirQuality.Good:
                return this.hap.Characteristic.AirQuality.GOOD;
            case winix_api_1.AirQuality.Fair:
                return this.hap.Characteristic.AirQuality.FAIR;
            case winix_api_1.AirQuality.Poor:
                return this.hap.Characteristic.AirQuality.POOR;
            default:
                return this.hap.Characteristic.AirQuality.UNKNOWN;
        }
    }
    shouldUseCachedValue(v) {
        return v !== undefined && Date.now() - this.lastWinixPoll < this.cacheIntervalMs;
    }
    polledWinix() {
        this.lastWinixPoll = Date.now();
    }
    toSwitch(plasmawave) {
        return plasmawave === winix_api_1.Plasmawave.On;
    }
    toPlasmawave(state) {
        return state ? winix_api_1.Plasmawave.On : winix_api_1.Plasmawave.Off;
    }
    registerService(service) {
        this.services.push(service);
        return service;
    }
    getServices() {
        return this.services;
    }
}
exports.WinixPurifierAccessory = WinixPurifierAccessory;
function assertError(error) {
    if (!(error instanceof Error)) {
        throw error;
    }
}
